/***********************************************************************************************************************
 *
 * forceTen - open source geography
 * ================================
 *
 * Copyright (C) 2007-2010 by Tidalwave s.a.s.
 * Project home page: http://forceten.tidalwave.it
 *
 ***********************************************************************************************************************
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License is distributed on
 * an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations under the License.
 *
 ***********************************************************************************************************************
 *
 * $Id: TileCacheDecorator.java,v c9bf78d7704e 2010/02/14 03:07:46 fabrizio $
 *
 **********************************************************************************************************************/
package it.tidalwave.geo.viewer.spi.jxmapviewer;

import javax.annotation.Nonnull;
import java.util.Locale;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.net.URI;
import java.awt.image.BufferedImage;
import javax.imageio.ImageIO;
import org.jdesktop.swingx.mapviewer.TileCache;
import it.tidalwave.util.NotFoundException;
import it.tidalwave.geo.viewer.spi.ResourceCache;

/***********************************************************************************************************************
 *
 * @author  Fabrizio Giudici
 * @version $Id: TileCacheDecorator.java,v c9bf78d7704e 2010/02/14 03:07:46 fabrizio $
 *
 **********************************************************************************************************************/
public class TileCacheDecorator extends TileCache
  {
    private final ResourceCache resourceCache;

    private final TileCache delegate;

    public TileCacheDecorator (final @Nonnull TileCache delegate, final @Nonnull ResourceCache resourceCache)
      {
        assert delegate != null : "null delegate";
        assert resourceCache != null : "null resourceCache";
        
        this.delegate = delegate;
        this.resourceCache = resourceCache;
      }
   
    @Override
    public BufferedImage get (final URI uri)
      throws IOException
      {
        BufferedImage image = delegate.get(uri);

        if (image == null)
          {
            try
              {
                image = resourceCache.findImage(uri);
                // Store to the memory delegate
                final ByteArrayOutputStream os = new ByteArrayOutputStream();
                String format = "JPEG";

                if (uri.toASCIIString().toLowerCase(Locale.ENGLISH).endsWith(".png"))
                  {
                    format = "PNG";
                  }

                ImageIO.write(image, format, os);
                delegate.put(uri, os.toByteArray(), image);
              }
            catch (NotFoundException e)
              {
                image = null;
              }
          }

        return image;
      }

    @Override
    public void put (final URI uri, final byte[] buffer, final BufferedImage image)
      {
        resourceCache.storeImage(uri, buffer);
        delegate.put(uri, buffer, image);
      }
  }
